@@ -4,7 +4,7 @@ use v5.10;
 use strict;
 use warnings;
 use bytes;
-use Socket qw(inet_ntop AF_INET AF_INET6);
+use Socket qw(inet_ntop inet_pton AF_INET AF_INET6);
 
 use constant {
     ATTR_CISCO_AVPAIR   => 1,
@@ -50,6 +50,11 @@ my %decode_map = (
     # abinary   => ...
 );
 
+if (!defined inet_pton(AF_INET6, '::1')) {
+    require Net::IP;
+    $decode_map{ipv6addr} = \&decode_ipv6addr_pp;
+}
+
 sub decode_string   { $_[0] }
 sub decode_int      { unpack('N', $_[0]) }
 sub decode_byte     { unpack('C', $_[0]) }
@@ -57,12 +62,26 @@ sub decode_short    { unpack('S>', $_[0]) }
 sub decode_signed   { unpack('l>', $_[0]) }
 sub decode_ipaddr   { inet_ntop(AF_INET, $_[0]) }
 sub decode_ipv6addr { inet_ntop(AF_INET6, $_[0]) }
+
+sub decode_ipv6addr_pp {
+    my $value = shift;
+
+    my $binary = unpack( 'B*', $value );
+    return undef if (! $binary);
+    my $ip_val = Net::IP::ip_bintoip( $binary, 6 );
+    return undef if (! $ip_val);
+    return Net::IP::ip_compress_address( $ip_val, 6 );
+}
+
 sub decode_octets   { '0x'.unpack("H*", $_[0]) }
 
 sub decode_combo_ip {
     my $ip = shift;
-    my $family = (length($ip) == 4) ? AF_INET : AF_INET6;
-    return inet_ntop($family, $ip);
+
+    if (length($ip) == 4) {
+        return $decode_map{ipaddr}->($ip);
+    }
+    return $decode_map{ipv6addr}->($ip);
 }
 
 sub decode_avpair {
